"""
Logging Root
"""

from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

__RCSID__ = "$Id$"

import logging
import time
import sys
import six
from DIRAC.FrameworkSystem.private.standardLogging.LogLevels import LogLevels
from DIRAC.FrameworkSystem.private.standardLogging.Logging import Logging
from DIRAC.Resources.LogBackends.StdoutBackend import StdoutBackend
from DIRAC.Core.Utilities import DIRACSingleton


@six.add_metaclass(DIRACSingleton.DIRACSingleton)
class LoggingRoot(Logging):
    """
    LoggingRoot is a particular Logging object: the first parent of the chain.

    - It is the one that initializes the root logger of the standard logging library and it configures it
    - As it defines the default behaviour of the Logging objects, it needs a specific class
    - It is unique, there is one and only one parent at the top of the chain: this justifies the usage of a Singleton

    """

    # Boolean preventing the LoggingRoot to be configured once more
    __configuredLogging = False

    def __init__(self):
        """
        Initialization of the LoggingRoot object.
        LoggingRoot :

        - initialize the UTC time
        - set the correct level defines by the user, or the default
        - add the custom level to logging: verbose, notice, always
        - register a default backend (stdout): all messages will be displayed here
        - update the format according to the command line argument

        """
        super(LoggingRoot, self).__init__()

        # this line removes some useless information from log records and improves the performances
        logging._srcfile = None  # pylint: disable=protected-access

        # initialize the root logger, which turns out to be a child of root, and disable propagation
        # to avoid any conflicts with external libs that would use "logging" too
        self._logger = logging.getLogger("dirac")
        self._logger.propagate = False

        # here we redefine the custom name to the empty string to remove the "\" in the display
        self._customName = ""

        # initialization of levels
        levels = LogLevels.getLevels()
        for level in levels:
            logging.addLevelName(levels[level], level)

        # root Logger level is set to NOTICE by default
        self._logger.setLevel(LogLevels.NOTICE)

        # initialization of the UTC time
        # Actually, time.gmtime is equal to UTC time: it has its DST flag to 0 which means there is no clock advance
        logging.Formatter.converter = time.gmtime

        # initialization of the default backend
        # use the StdoutBackend directly to avoid dependancy loop with ObjectLoader
        self._addBackend(StdoutBackend)

        # configuration of the level and update the format
        self.__configureLevel()

    def initialize(self, systemName, cfgPath, forceInit=False):
        """
        Configure the root Logging.
        It can be possible to :
        - attach it some backends : LogBackends = stdout,stderr,file,server
        - attach backend options : BackendOptions { FileName = /tmp/file.log }
        - add colors and the path of the call : LogColor = True, LogShowLine = True
        - precise a level : LogLevel = DEBUG

        :param str systemName: <system name>/<component name>
        :param str cfgPath: configuration path
        :param bool forceInit: Force the initialization even if it had already happened.
                               This should not be used !! The only case is LocalConfiguration.enableCS
                               In order to take into account extensions' backends
        """
        # we have to put the import line here to avoid a dependancy loop
        from DIRAC import gConfig

        self._lockConfig.acquire()
        try:
            if not LoggingRoot.__configuredLogging or forceInit:
                Logging._componentName = systemName

                # prepare to remove all the backends from the root Logging as in the old gLogger.
                # store them in a list handlersToRemove.
                # we will remove them later, because some components as ObjectLoader need a backend.
                # this can be useful to have logs only in a file for instance.
                handlersToRemove = []
                for backend in self._backendsList:
                    handlersToRemove.append(backend.getHandler())
                del self._backendsList[:]

                # get the backends, the backend options and add them to the root Logging
                desiredBackends = self.__getBackendsFromCFG(cfgPath)
                for backend in desiredBackends:
                    desiredOptions = self.__getBackendOptionsFromCFG(cfgPath, backend)
                    self.registerBackend(desiredOptions.get("Plugin", backend), desiredOptions)

                # Format options
                self._options["color"] = gConfig.getValue("%s/LogColor" % cfgPath, False)

                # Remove the old backends
                for handler in handlersToRemove:
                    self._logger.removeHandler(handler)

                levelName = gConfig.getValue("%s/LogLevel" % cfgPath, None)
                if levelName is not None:
                    self.setLevel(levelName)

                LoggingRoot.__configuredLogging = True
        finally:
            self._lockConfig.release()

    def __getBackendsFromCFG(self, cfgPath):
        """
        Get backends from the configuration and register them in LoggingRoot.
        This is the new way to get the backends providing a general configuration.

        :param str cfgPath: configuration path
        """
        # We have to put the import line here to avoid a dependancy loop
        from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations
        from DIRAC import gConfig

        # get the second last string representing the component type in the configuration
        # example : 'Agents', 'Services'
        component = cfgPath.split("/")[-2]
        operation = Operations()

        # Search desired backends in the component
        desiredBackends = gConfig.getValue("%s/%s" % (cfgPath, "LogBackends"), [])
        if not desiredBackends:
            # Search desired backends in the operation section according to the
            # component type
            desiredBackends = operation.getValue("Logging/Default%sBackends" % component, [])
            if not desiredBackends:
                # Search desired backends in the operation section
                desiredBackends = operation.getValue("Logging/DefaultBackends", [])
                if not desiredBackends:
                    # Default value
                    desiredBackends = ["stdout"]

        return desiredBackends

    def __getBackendOptionsFromCFG(self, cfgPath, backend):
        """
        Get backend options from the configuration.

        :param cfgPath: configuration path
        :param backend: backend identifier: stdout, file, f04
        """
        # We have to put the import lines here to avoid a dependancy loop
        from DIRAC import gConfig
        from DIRAC.ConfigurationSystem.Client.Helpers.Resources import getBackendConfig

        backendOptions = {}
        # Search backends config in the resources section
        retDictRessources = getBackendConfig(backend)
        if retDictRessources["OK"]:
            backendOptions = retDictRessources["Value"]

        # Search backends config in the component to update some options
        retDictConfig = gConfig.getOptionsDict("%s/%s/%s" % (cfgPath, "LogBackendsConfig", backend))
        if retDictConfig["OK"]:
            backendOptions.update(retDictConfig["Value"])

        return backendOptions

    def __configureLevel(self):
        """
        Configure the log level of the root Logging according to the argv parameter
        It can be : -d, -dd, -ddd
        Work only for clients, scripts and tests
        Configuration/Client/LocalConfiguration manages services,agents and executors
        """
        debLevs = 0
        for arg in sys.argv:
            if arg.find("-d") == 0:
                debLevs += arg.count("d")
        if debLevs == 1:
            self.setLevel("verbose")
        elif debLevs == 2:
            self.setLevel("verbose")
            self.showHeaders(True)
        elif debLevs >= 3:
            self.setLevel("debug")
            self.showHeaders(True)
            self.showThreadIDs(True)

    def enableLogsFromExternalLibs(self):
        """
        Enable the display of the logs coming from external libraries
        """
        self.__enableLogsFromExternalLibs()

    def disableLogsFromExternalLibs(self):
        """
        Disable the display of the logs coming from external libraries
        """
        self.__enableLogsFromExternalLibs(False)

    @staticmethod
    def __enableLogsFromExternalLibs(isEnabled=True):
        """
        Configure the root logger from 'logging' for an external library use.
        By default the root logger is configured with:

        - debug level,
        - stderr output
        - custom format close to the DIRAC format

        :param bool isEnabled: allows logs from external libs
        """
        rootLogger = logging.getLogger()
        rootLogger.handlers = []
        if isEnabled:
            logging.basicConfig(
                level=logging.DEBUG,
                format="%(asctime)s UTC ExternalLibrary/%(name)s %(levelname)s: %(message)s",
                datefmt="%Y-%m-%d %H:%M:%S",
            )
        else:
            rootLogger.addHandler(logging.NullHandler())



""" Test for StorageManagement clients
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

# pylint: disable=protected-access,missing-docstring,invalid-name

import unittest
from mock import MagicMock, patch

from DIRAC import S_OK, S_ERROR
from DIRAC.StorageManagementSystem.Client.StorageManagerClient import getFilesToStage
from DIRAC.DataManagementSystem.Client.test.mock_DM import dm_mock
import errno

# Use side_effect instead of return_value so the dict in the result is copied
mockObjectSE1 = MagicMock()
mockObjectSE1.getFileMetadata.side_effect = lambda *_: S_OK(
    {
        "Successful": {"/a/lfn/1.txt": {"Accessible": False}},
        "Failed": {},
    }
)
mockObjectSE1.getStatus.side_effect = lambda *_: S_OK({"DiskSE": False, "TapeSE": True})

mockObjectSE2 = MagicMock()
mockObjectSE2.getFileMetadata.side_effect = lambda *_: S_OK(
    {
        "Successful": {"/a/lfn/2.txt": {"Cached": 1, "Accessible": True}},
        "Failed": {},
    }
)
mockObjectSE2.getStatus.side_effect = lambda *_: S_OK({"DiskSE": False, "TapeSE": True})

mockObjectSE3 = MagicMock()
mockObjectSE3.getFileMetadata.side_effect = lambda *_: S_OK(
    {
        "Successful": {},
        "Failed": {"/a/lfn/2.txt": "error"},
    }
)
mockObjectSE3.getStatus.side_effect = lambda *_: S_OK({"DiskSE": False, "TapeSE": True})

mockObjectSE4 = MagicMock()
mockObjectSE4.getFileMetadata.side_effect = lambda *_: S_OK(
    {"Successful": {}, "Failed": {"/a/lfn/2.txt": S_ERROR(errno.ENOENT, "")["Message"]}}
)
mockObjectSE4.getStatus.side_effect = lambda *_: S_OK({"DiskSE": False, "TapeSE": True})

mockObjectSE5 = MagicMock()
mockObjectSE5.getFileMetadata.side_effect = lambda *_: S_OK(
    {"Successful": {"/a/lfn/1.txt": {"Accessible": False}}, "Failed": {}}
)
mockObjectSE5.getStatus.side_effect = lambda *_: S_OK({"DiskSE": True, "TapeSE": False})

mockObjectSE6 = MagicMock()
mockObjectSE6.getFileMetadata.side_effect = lambda *_: S_OK(
    {"Successful": {"/a/lfn/2.txt": {"Cached": 0, "Accessible": False}}, "Failed": {}}
)
mockObjectSE6.getStatus.side_effect = lambda *_: S_OK({"DiskSE": False, "TapeSE": True})


mockObjectDMSHelper = MagicMock()
mockObjectDMSHelper.getLocalSiteForSE.side_effect = lambda *_: S_OK("mySite")
mockObjectDMSHelper.getSitesForSE.side_effect = lambda *_: S_OK(["mySite"])


class ClientsTestCase(unittest.TestCase):
    """Base class for the clients test cases"""

    def setUp(self):
        from DIRAC import gLogger

        gLogger.setLevel("DEBUG")

    def tearDown(self):
        pass


#############################################################################


class StorageManagerSuccess(ClientsTestCase):
    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.DataManager", return_value=dm_mock)
    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.StorageElement", return_value=mockObjectSE1)
    def test_getFilesToStage_withFilesToStage(self, _patch, _patched):
        """Test where the StorageElement mock will return files offline"""
        res = getFilesToStage(["/a/lfn/1.txt"], checkOnlyTapeSEs=False)
        self.assertTrue(res["OK"])
        self.assertEqual(res["Value"]["onlineLFNs"], [])
        self.assertIn(res["Value"]["offlineLFNs"], [{"SE1": ["/a/lfn/1.txt"]}, {"SE2": ["/a/lfn/1.txt"]}])
        self.assertEqual(res["Value"]["absentLFNs"], {})
        self.assertEqual(res["Value"]["failedLFNs"], [])

    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.DataManager", return_value=dm_mock)
    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.StorageElement", return_value=mockObjectSE2)
    def test_getFilesToStage_noFilesToStage(self, _patch, _patched):
        """Test where the StorageElement mock will return files online"""
        res = getFilesToStage(["/a/lfn/2.txt"], checkOnlyTapeSEs=False)
        self.assertTrue(res["OK"])
        self.assertEqual(res["Value"]["onlineLFNs"], ["/a/lfn/2.txt"])
        self.assertEqual(res["Value"]["offlineLFNs"], {})
        self.assertEqual(res["Value"]["absentLFNs"], {})
        self.assertEqual(res["Value"]["failedLFNs"], [])

    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.DataManager", return_value=dm_mock)
    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.StorageElement", return_value=mockObjectSE3)
    def test_getFilesToStage_seErrors(self, _patch, _patched):
        """Test where the StorageElement will return failure"""
        res = getFilesToStage(["/a/lfn/2.txt"], checkOnlyTapeSEs=False)
        self.assertTrue(res["OK"])
        self.assertEqual(res["Value"]["onlineLFNs"], [])
        self.assertEqual(res["Value"]["offlineLFNs"], {})
        self.assertEqual(res["Value"]["absentLFNs"], {})
        self.assertEqual(res["Value"]["failedLFNs"], ["/a/lfn/2.txt"])

    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.DataManager", return_value=dm_mock)
    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.StorageElement", return_value=mockObjectSE4)
    def test_getFilesToStage_noSuchFile(self, _patch, _patched):
        """Test where the StorageElement will return file is absent"""
        res = getFilesToStage(["/a/lfn/2.txt"], checkOnlyTapeSEs=False)
        self.assertTrue(res["OK"])
        self.assertEqual(res["Value"]["onlineLFNs"], [])
        self.assertEqual(res["Value"]["offlineLFNs"], {})
        self.assertEqual(
            res["Value"]["absentLFNs"], {"/a/lfn/2.txt": "No such file or directory ( 2 : File not at SE1,SE2)"}
        )
        self.assertEqual(res["Value"]["failedLFNs"], [])

    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.DataManager", return_value=dm_mock)
    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.StorageElement", return_value=mockObjectSE5)
    def test_getFilesToStage_fileInaccessibleAtDisk(self, _patch, _patched):
        """Test where the StorageElement will return file is unavailable at a Disk SE"""
        res = getFilesToStage(["/a/lfn/1.txt"], checkOnlyTapeSEs=False)
        self.assertTrue(res["OK"])
        self.assertEqual(res["Value"]["onlineLFNs"], [])
        self.assertEqual(res["Value"]["offlineLFNs"], {})
        self.assertEqual(res["Value"]["absentLFNs"], {})
        self.assertEqual(res["Value"]["failedLFNs"], ["/a/lfn/1.txt"])

    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.DataManager", return_value=dm_mock)
    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.StorageElement", return_value=mockObjectSE2)
    def test_getFilesToStage_tapeSEOnly_1(self, _patch, _patched):
        """Test where the StorageElement will return file is available"""
        res = getFilesToStage(["/a/lfn/2.txt"], checkOnlyTapeSEs=True)
        self.assertTrue(res["OK"])
        self.assertEqual(res["Value"]["onlineLFNs"], ["/a/lfn/2.txt"])
        self.assertEqual(res["Value"]["offlineLFNs"], {})
        self.assertEqual(res["Value"]["absentLFNs"], {})
        self.assertEqual(res["Value"]["failedLFNs"], [])

    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.DataManager", return_value=dm_mock)
    @patch("DIRAC.StorageManagementSystem.Client.StorageManagerClient.StorageElement", return_value=mockObjectSE6)
    def test_getFilesToStage_tapeSEOnly_2(self, _patch, _patched):
        """Test where the StorageElement will return file is at offline at tape"""

        with patch(
            "DIRAC.StorageManagementSystem.Client.StorageManagerClient.random.choice",
            new=MagicMock(return_value="SERandom"),
        ):
            res = getFilesToStage(["/a/lfn/2.txt"], checkOnlyTapeSEs=True)
        self.assertTrue(res["OK"])
        self.assertEqual(res["Value"]["onlineLFNs"], [])
        self.assertEqual(res["Value"]["offlineLFNs"], {"SERandom": ["/a/lfn/2.txt"]})
        self.assertEqual(res["Value"]["absentLFNs"], {})
        self.assertEqual(res["Value"]["failedLFNs"], [])


if __name__ == "__main__":
    suite = unittest.defaultTestLoader.loadTestsFromTestCase(ClientsTestCase)
    suite.addTest(unittest.defaultTestLoader.loadTestsFromTestCase(StorageManagerSuccess))
    testResult = unittest.TextTestRunner(verbosity=2).run(suite)



""" X509Request is a class for managing X509 requests with their Pkeys.
It's main use is for proxy delegation.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import M2Crypto
from DIRAC import S_OK, S_ERROR
from DIRAC.Core.Utilities import DErrno

# from DIRAC.Core.Security.m2crypto.X509Chain import X509Chain  # pylint: disable=import-error

# pylint: disable=broad-except


class X509Request(object):
    """
    Class representing X509 Certificate Request. it is used for delegation.
    Please see :ref:`about_proxies` for detailed explanations on delegation,
    and :py:class:`DIRAC.Core.Security.m2crypto.X509Chain` for code examples.

    """

    def __init__(self, reqObj=None, pkeyObj=None):
        """C'tor

        :param reqObj: M2Crypto.X509.Request object. Never used. Shall be removed
        :param pkeyObj: M2Crypto.EVP.PKey() object. Never used. Shall be removed
        """
        self.__valid = False
        self.__reqObj = reqObj
        self.__pkeyObj = pkeyObj
        if reqObj and pkeyObj:  # isn't it a bit too liberal?
            self.__valid = True

    def generateProxyRequest(self, bitStrength=1024, limited=False):
        """
        Initialize the Request object as well as the PKey.

        :param bitStrength: (default 1024) length of the key generated
        :param limited: (default False) If True, request is done for a limited proxy
        """
        # self.__pkeyObj is both the public and private key
        self.__pkeyObj = M2Crypto.EVP.PKey()
        self.__pkeyObj.assign_rsa(
            M2Crypto.RSA.gen_key(bitStrength, 65537, callback=M2Crypto.util.quiet_genparam_callback)
        )
        self.__reqObj = M2Crypto.X509.Request()
        self.__reqObj.set_pubkey(self.__pkeyObj)

        if limited:
            self.__reqObj.get_subject().add_entry_by_txt(
                field="CN", type=M2Crypto.ASN1.MBSTRING_ASC, entry="limited proxy", len=-1, loc=-1, set=0
            )
        else:
            self.__reqObj.get_subject().add_entry_by_txt(
                field="CN", type=M2Crypto.ASN1.MBSTRING_ASC, entry="proxy", len=-1, loc=-1, set=0
            )
        self.__reqObj.sign(self.__pkeyObj, "sha256")
        self.__valid = True

    def dumpRequest(self):
        """
        Get the request as a string

        :returns: S_OK(pem encoded request)
        """
        if not self.__valid:
            return S_ERROR(DErrno.ENOCERT)
        try:
            reqStr = self.__reqObj.as_pem()
        except Exception as e:
            return S_ERROR(DErrno.EX509, "Can't serialize request: %s" % e)
        return S_OK(reqStr)

    # def getRequestObject(self):
    #   """
    #   Get internal X509Request object
    #   Not used
    #   """
    #   return S_OK(self.__reqObj)

    def getPKey(self):
        """
        Get PKey Internal

        :returns: M2Crypto.EVP.PKEY object
        """
        return self.__pkeyObj

    def dumpPKey(self):
        """
        Get the private as a string

        :returns: S_OK(PEM encoded PKey)
        """
        if not self.__valid:
            return S_ERROR(DErrno.ENOCERT)
        try:
            pkeyStr = self.__pkeyObj.as_pem(cipher=None, callback=M2Crypto.util.no_passphrase_callback)
        except Exception as e:
            return S_ERROR(DErrno.EX509, "Can't serialize pkey: %s" % e)
        return S_OK(pkeyStr)

    def dumpAll(self):
        """
        Dump the Request and the PKey into a string

        :returns: S_OK(PEM encoded req + key), S_ERROR
        """

        if not self.__valid:
            return S_ERROR(DErrno.ENOCERT)

        req = self.dumpRequest()
        pkey = self.dumpPKey()
        if not req["OK"]:
            return S_ERROR(DErrno.EX509, "Can't serialize request: %s" % req["Message"])
        if not pkey["OK"]:
            return S_ERROR(DErrno.EX509, "Can't serialize pkey: %s" % pkey["Message"])
        return S_OK(b"%s%s" % (req["Value"], pkey["Value"]))

    def loadAllFromString(self, pemData):
        """load the Request and key argument from a PEM encoded string.

        :param pemData: PEN encoded string containing Request and PKey

        :returns: S_OK()
        """
        try:
            self.__reqObj = M2Crypto.X509.load_request_string(pemData)
        except Exception as e:
            return S_ERROR(DErrno.ENOCERT, str(e))
        try:
            self.__pkeyObj = M2Crypto.EVP.load_key_string(pemData)
        except Exception as e:
            return S_ERROR(DErrno.ENOPKEY, str(e))
        self.__valid = True
        return S_OK()

    # def generateChainFromResponse(self, pemData):
    #   """
    #   Generate a X509 Chain from the pkey and the pem data passed as the argument
    #   Return : S_OK( X509Chain ) / S_ERROR
    #   """
    #   if not self.__valid:
    #     return S_ERROR(DErrno.ENOCERT)
    #   chain = X509Chain()
    #   ret = chain.loadChainFromString(pemData)
    #   if not ret['OK']:
    #     return ret
    #   ret = chain.setPKey(self.__pkeyObj)
    #   if not ret['OK']:
    #     return ret
    #   return chain

    def getSubjectDN(self):
        """
        Get subject DN of the request as a string

        :return: S_OK( string )/S_ERROR
        """
        if not self.__valid:
            return S_ERROR(DErrno.ENOCERT)
        return S_OK(str(self.__reqObj.get_subject()))

    def checkChain(self, chain):
        """
        Check that the public keys of the chain and the request match.

        :param chain: :py:class:`X509Chain` object
        """

        if not self.__valid:
            return S_ERROR(DErrno.ENOCERT)
        retVal = chain.getCertInChain()
        if not retVal["OK"]:
            return retVal
        lastCert = retVal["Value"]
        chainPubKey = lastCert.getPublicKey()
        if not chainPubKey["OK"]:
            return chainPubKey

        # as_der will dump public key info, while as_pem
        # dumps private key.
        chainPubKey = chainPubKey["Value"].as_der()
        reqPubKey = self.__reqObj.get_pubkey().as_der()
        if not chainPubKey == reqPubKey:
            return S_ERROR(DErrno.EX509, "Public keys do not match")

        return S_OK(True)



# pylint: disable=protected-access
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

from DIRAC.AccountingSystem.Client.Types.DataOperation import DataOperation
from DIRAC.AccountingSystem.Client.Types.StorageOccupancy import StorageOccupancy


def createDataOperationAccountingRecord():
    accountingDict = {}
    accountingDict["OperationType"] = "putAndRegister"
    accountingDict["User"] = "system"
    accountingDict["Protocol"] = "DataManager"
    accountingDict["RegistrationTime"] = 0.0
    accountingDict["RegistrationOK"] = 0
    accountingDict["RegistrationTotal"] = 0
    accountingDict["Destination"] = "se"
    accountingDict["TransferTotal"] = 1
    accountingDict["TransferOK"] = 1
    accountingDict["TransferSize"] = 1
    accountingDict["TransferTime"] = 0.0
    accountingDict["FinalStatus"] = "Successful"
    accountingDict["Source"] = "testSite"
    oDataOperation = DataOperation()
    oDataOperation.setValuesFromDict(accountingDict)
    return oDataOperation


def createStorageOccupancyAccountingRecord():
    accountingDict = {}
    accountingDict["Site"] = "LCG.PIPPO.it"
    accountingDict["Endpoint"] = "somewhere.in.topolinea.it"
    accountingDict["StorageElement"] = "PIPPO-SE"
    accountingDict["SpaceType"] = "Total"
    accountingDict["Space"] = 123456
    oSO = StorageOccupancy()
    oSO.setValuesFromDict(accountingDict)
    return oSO



""" The SystemAdministratorClient is a class representing the client of the DIRAC
    SystemAdministrator service. It has also methods to update the Configuration
    Service with the DIRAC components options
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.Base.Client import Client, createClient

SYSADMIN_PORT = 9162


@createClient("Framework/SystemAdministrator")
class SystemAdministratorClient(Client):
    def __init__(self, host, port=None, **kwargs):
        """Constructor function. Takes a mandatory host parameter"""
        super(SystemAdministratorClient, self).__init__(**kwargs)
        if not port:
            port = SYSADMIN_PORT
        self.setServer("dips://%s:%s/Framework/SystemAdministrator" % (host, port))



"""
This class is used to define the plot using the plot attributes.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from DIRAC import S_OK, gLogger

from DIRAC.MonitoringSystem.Client.Types.ComponentMonitoring import ComponentMonitoring
from DIRAC.MonitoringSystem.private.Plotters.BasePlotter import BasePlotter

__RCSID__ = "$Id$"


class ComponentMonitoringPlotter(BasePlotter):

    """
    .. class:: ComponentMonitoringPlotter

    It is used to create the plots.

    param: str _typeName monitoring type
    param: list _typeKeyFields list of keys what we monitor (list of attributes)
    """

    _typeName = "ComponentMonitoring"
    _typeKeyFields = ComponentMonitoring().keyFields

    def __reportAllResources(self, reportRequest, metric, unit):

        retVal = self._getTimedData(
            startTime=reportRequest["startTime"],
            endTime=reportRequest["endTime"],
            selectField=metric,
            preCondDict=reportRequest["condDict"],
            metadataDict={"DynamicBucketing": False, "metric": "avg"},
        )
        if not retVal["OK"]:
            return retVal

        dataDict, granularity = retVal["Value"]
        try:
            _, _, _, unitName = self._findSuitableUnit(dataDict, self._getAccumulationMaxValue(dataDict), unit)
        except AttributeError as e:
            gLogger.warn(e)
            unitName = unit

        return S_OK({"data": dataDict, "granularity": granularity, "unit": unitName})

    def __plotAllResources(self, reportRequest, plotInfo, filename, title):
        metadata = {
            "title": "%s by %s" % (title, reportRequest["grouping"]),
            "starttime": reportRequest["startTime"],
            "endtime": reportRequest["endTime"],
            "span": plotInfo["granularity"],
            "skipEdgeColor": True,
            "ylabel": plotInfo["unit"],
        }

        plotInfo["data"] = self._fillWithZero(
            granularity=plotInfo["granularity"],
            startEpoch=reportRequest["startTime"],
            endEpoch=reportRequest["endTime"],
            dataDict=plotInfo["data"],
        )

        return self._generateStackedLinePlot(filename=filename, dataDict=plotInfo["data"], metadata=metadata)

    _reportRunningThreadsName = "Number of running threads"

    def _reportRunningThreads(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "threads", "threads")

    def _plotRunningThreads(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Number of running threads")

    _reportCpuUsageName = "CPU usage"

    def _reportCpuUsage(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return: S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "cpuPercentage", "percentage")

    def _plotCpuUsage(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "CPU usage")

    _reportMemoryUsageName = "Memory usage"

    def _reportMemoryUsage(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return: S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "memoryUsage", "bytes")

    def _plotMemoryUsage(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Memory usage")

    _reportRunningTimeName = "Running time"

    def _reportRunningTime(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return: S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "runningTime", "time")

    def _plotRunningTime(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Running time")

    _reportConnectionsName = "Number of Connections"

    def _reportConnections(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return: S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "Connections", "Connections")

    def _plotConnections(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Number of Connections")

    _reportActiveQueriesName = "Number of ActiveQueries"

    def _reportActiveQueries(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return: S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "ActiveQueries", "ActiveQueries")

    def _plotActiveQueries(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Number of ActiveQueries")

    _reportPendingQueriesName = "Number of PendingQueries"

    def _reportPendingQueries(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return: S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "PendingQueries", "PendingQueries")

    def _plotPendingQueries(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Number of PendingQueries")

    _reportMaxFDName = "Max File Descriptors"

    def _reportMaxFD(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return: S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "MaxFD", "MaxFD")

    def _plotMaxFD(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Max File Descriptors")

    _reportActivityRunningThreadsName = "Number of activity running threads(activity)"

    def _reportActivityRunningThreads(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "RunningThreads", "RunningThreads")

    def _plotActivityRunningThreads(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Number of running threads(activity)")

    _reportServiceResponseTimeName = "Service response time"

    def _reportServiceResponseTime(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "ServiceResponseTime", "seconds")

    def _plotServiceResponseTime(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Service response time")

    _reportAgentCycleDurationName = "Agent cycle duration"

    def _reportAgentCycleDuration(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "cycleDuration", "seconds")

    def _plotAgentCycleDuration(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Agent cycle duration")

    _reportAgentCyclesName = "Number of agent cycles"

    def _reportAgentCycles(self, reportRequest):
        """It is used to retrieve the data from the database.

        :param dict reportRequest: contains attributes used to create the plot.
        :return S_OK or S_ERROR {'data':value1, 'granularity':value2} value1 is a dictionary, value2 is the bucket length
        """
        return self.__reportAllResources(reportRequest, "cycles", "AgentCycles")

    def _plotAgentCycles(self, reportRequest, plotInfo, filename):
        """It creates the plot.

        :param dict reportRequest: plot attributes
        :param dict plotInfo: contains all the data which are used to create the plot
        :param str filename:
        :return: S_OK or S_ERROR { 'plot' : value1, 'thumbnail' : value2 } value1 and value2 are TRUE/FALSE
        """
        return self.__plotAllResources(reportRequest, plotInfo, filename, "Number of agent cycles")



#!/usr/bin/env python
"""
Command line tool to remove local and remote proxies

Example:
  $ dirac-proxy-destroy -a
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import os

import DIRAC
from DIRAC import gLogger, S_OK
from DIRAC.Core.Utilities.DIRACScript import DIRACScript as Script

from DIRAC.Core.Security import Locations, ProxyInfo
from DIRAC.Core.DISET.RPCClient import RPCClient
from DIRAC.FrameworkSystem.Client.ProxyManagerClient import gProxyManager
from DIRAC.ConfigurationSystem.Client.Helpers import Registry


class Params(object):
    """
    handles input options for dirac-proxy-destroy
    """

    def __init__(self):
        """
        creates a Params class with default values
        """
        self.vos = []
        self.delete_all = False

    def addVO(self, voname):
        """
        adds a VO to be deleted from remote proxies
        """
        self.vos.append(voname)
        return S_OK()

    def setDeleteAll(self, _):
        """
        deletes local and remote proxies
        """
        self.delete_all = True
        return S_OK()

    def needsValidProxy(self):
        """
        returns true if any remote operations are required
        """
        return self.vos or self.delete_all

    # note the magic : and =
    def registerCLISwitches(self):
        """
        add options to dirac option parser
        """
        Script.registerSwitch(
            "a", "all", "Delete the local and all uploaded proxies (the nuclear option)", self.setDeleteAll
        )
        Script.registerSwitch("v:", "vo=", "Delete uploaded proxy for vo name given", self.addVO)


def getProxyGroups():
    """
    Returns a set of all remote proxy groups stored on the dirac server for the user invoking the command.
    """
    proxies = gProxyManager.getUserProxiesInfo()
    if not proxies["OK"]:
        raise RuntimeError("Could not retrieve uploaded proxy info.")

    user_groups = set()
    for dn in proxies["Value"]:
        dn_groups = set(proxies["Value"][dn].keys())
        user_groups.update(dn_groups)

    return user_groups


def mapVoToGroups(voname):
    """
    Returns all groups available for a given VO as a set.
    """

    vo_dict = Registry.getGroupsForVO(voname)
    if not vo_dict["OK"]:
        raise RuntimeError("Could not retrieve groups for vo %s." % voname)

    return set(vo_dict["Value"])


def deleteRemoteProxy(userdn, vogroup):
    """
    Deletes proxy for a vogroup for the user envoking this function.
    Returns a list of all deleted proxies (if any).
    """
    rpcClient = RPCClient("Framework/ProxyManager")
    retVal = rpcClient.deleteProxyBundle([(userdn, vogroup)])

    if retVal["OK"]:
        gLogger.notice("Deleted proxy for %s." % vogroup)
    else:
        gLogger.error("Failed to delete proxy for %s." % vogroup)


def deleteLocalProxy(proxyLoc):
    """
    Deletes the local proxy.
    Returns false if no local proxy found.
    """
    try:
        os.unlink(proxyLoc)
    except IOError:
        gLogger.error("IOError: Failed to delete local proxy.")
        return
    except OSError:
        gLogger.error("OSError: Failed to delete local proxy.")
        return
    gLogger.notice("Local proxy deleted.")


def run():
    """
    main program entry point
    """
    options = Params()
    options.registerCLISwitches()

    Script.parseCommandLine(ignoreErrors=True)

    if options.delete_all and options.vos:
        gLogger.error("-a and -v options are mutually exclusive. Please pick one or the other.")
        return 1

    proxyLoc = Locations.getDefaultProxyLocation()

    if not os.path.exists(proxyLoc):
        gLogger.error("No local proxy found in %s, exiting." % proxyLoc)
        return 1

    result = ProxyInfo.getProxyInfo(proxyLoc, True)
    if not result["OK"]:
        raise RuntimeError("Failed to get local proxy info.")

    if result["Value"]["secondsLeft"] < 60 and options.needsValidProxy():
        raise RuntimeError("Lifetime of local proxy too short, please renew proxy.")

    userDN = result["Value"]["identity"]

    if options.delete_all:
        # delete remote proxies
        remote_groups = getProxyGroups()
        if not remote_groups:
            gLogger.notice("No remote proxies found.")
        for vo_group in remote_groups:
            deleteRemoteProxy(userDN, vo_group)
        # delete local proxy
        deleteLocalProxy(proxyLoc)
    elif options.vos:
        vo_groups = set()
        for voname in options.vos:
            vo_groups.update(mapVoToGroups(voname))
        # filter set of all groups to only contain groups for which there is a user proxy
        user_groups = getProxyGroups()
        vo_groups.intersection_update(user_groups)
        if not vo_groups:
            gLogger.notice("You have no proxies registered for any of the specified VOs.")
        for group in vo_groups:
            deleteRemoteProxy(userDN, group)
    else:
        deleteLocalProxy(proxyLoc)

    return 0


@Script()
def main():
    try:
        retval = run()
        DIRAC.exit(retval)
    except RuntimeError as rtError:
        gLogger.error("Operation failed: %s" % str(rtError))
    DIRAC.exit(1)


if __name__ == "__main__":
    main()



""" InfoGetter

  Module used to map the policies with the CS.

"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

import copy

from DIRAC import S_OK, S_ERROR, gConfig, gLogger
from DIRAC.ResourceStatusSystem.Utilities import RssConfiguration, Utils


def getPoliciesThatApply(decisionParams):
    """
    Method that sanitizes the input parameters and returns the policies that
    match them. Matches the input dictionary with the policies configuration in
    the CS. It returns a list of policy dictionaries that matched.
    """

    # InfoGetter is being called from SiteInspector Agent

    decisionParams = _sanitizedecisionParams(decisionParams)
    gLogger.debug("Sanitized decisionParams: %s" % str(decisionParams))

    policiesThatApply = []

    # Get policies configuration metadata from CS.
    policiesConfig = RssConfiguration.getPolicies()
    if not policiesConfig["OK"]:
        return policiesConfig
    policiesConfig = policiesConfig["Value"]
    gLogger.debug("All policies: %s" % str(policiesConfig))

    # Each policy, has the following format
    # <policyName>
    # \
    #  policyType = <policyType>
    #  matchParams
    #  \
    #   ...
    #  configParams
    #  \
    #   ...

    # Get policies that match the given decisionParameters
    for policyName, policySetup in policiesConfig.items():

        # The parameter policyType replaces policyName, so if it is not present,
        # we pick policyName
        try:
            policyType = policySetup["policyType"][0]
        except KeyError:
            policyType = policyName
            # continue

        # The section matchParams is not mandatory, so we set {} as default.
        policyMatchParams = policySetup.get("matchParams", {})
        gLogger.debug("matchParams of %s: %s" % (policyName, str(policyMatchParams)))

        # FIXME: make sure the values in the policyConfigParams dictionary are typed !!
        policyConfigParams = {}
        # policyConfigParams = policySetup.get( 'configParams', {} )
        policyMatch = Utils.configMatch(decisionParams, policyMatchParams)
        gLogger.debug("PolicyMatch for decisionParams %s: %s" % (decisionParams, str(policyMatch)))

        # WARNING: we need an additional filtering function when the matching
        # is not straightforward (e.g. when the policy specify a 'domain', while
        # the decisionParams has only the name of the element)
        if policyMatch and _filterPolicies(decisionParams, policyMatchParams):
            policiesThatApply.append((policyName, policyType, policyConfigParams))

    gLogger.debug("policies that apply (before post-processing): %s" % str(policiesThatApply))
    policiesThatApply = postProcessingPolicyList(policiesThatApply)
    gLogger.debug("policies that apply (after post-processing): %s" % str(policiesThatApply))

    policiesToBeLoaded = []
    # Gets policies parameters from code.
    for policyName, policyType, _policyConfigParams in policiesThatApply:

        try:
            configModule = Utils.voimport("DIRAC.ResourceStatusSystem.Policy.Configurations")
            policies = copy.deepcopy(configModule.POLICIESMETA)
            policyMeta = policies[policyType]
        except KeyError:
            continue

        # We are not going to use name / type anymore, but we keep them for debugging
        # and future usage.
        policyDict = {"name": policyName, "type": policyType, "args": {}}

        # args is one of the parameters we are going to use on the policies. We copy
        # the defaults and then we update if with whatever comes from the CS.
        policyDict.update(policyMeta)

        policiesToBeLoaded.append(policyDict)

    return S_OK(policiesToBeLoaded)


def getPolicyActionsThatApply(decisionParams, singlePolicyResults, policyCombinedResults):
    """
    Method that sanitizes the input parameters and returns the policies actions
    that match them. Matches the input dictionary with the policy actions
    configuration in the CS. It returns a list of policy actions names that
    matched.
    """

    decisionParams = _sanitizedecisionParams(decisionParams)

    policyActionsThatApply = []

    # Get policies configuration metadata from CS.
    policyActionsConfig = RssConfiguration.getPolicyActions()
    if not policyActionsConfig["OK"]:
        return policyActionsConfig
    policyActionsConfig = policyActionsConfig["Value"]

    # Let's create a dictionary to use it with configMatch
    policyResults = {}
    for policyResult in singlePolicyResults:
        try:
            policyResults[policyResult["Policy"]["name"]] = policyResult["Status"]
        except KeyError:
            continue

    # Get policies that match the given decissionParameters
    for policyActionName, policyActionConfig in policyActionsConfig.items():

        # The parameter policyType is mandatory. If not present, we pick policyActionName
        try:
            policyActionType = policyActionConfig["actionType"][0]
        except KeyError:
            policyActionType = policyActionName
            # continue

        # We get matchParams to be compared against decisionParams
        policyActionMatchParams = policyActionConfig.get("matchParams", {})
        policyMatch = Utils.configMatch(decisionParams, policyActionMatchParams)
        # policyMatch = Utils.configMatch( decisionParams, policyActionConfig )
        if not policyMatch:
            continue

        # Let's check single policy results
        # Assumed structure:
        # ...
        # policyResults
        # <PolicyName> = <PolicyResult1>,<PolicyResult2>...
        policyActionPolicyResults = policyActionConfig.get("policyResults", {})
        policyResultsMatch = Utils.configMatch(policyResults, policyActionPolicyResults)
        if not policyResultsMatch:
            continue

        # combinedResult
        # \Status = X,Y
        # \Reason = asdasd,asdsa
        policyActionCombinedResult = policyActionConfig.get("combinedResult", {})
        policyCombinedMatch = Utils.configMatch(policyCombinedResults, policyActionCombinedResult)
        if not policyCombinedMatch:
            continue

        # policyActionsThatApply.append( policyActionName )
        # They may not be necessarily the same
        policyActionsThatApply.append((policyActionName, policyActionType))

    return S_OK(policyActionsThatApply)


def _sanitizedecisionParams(decisionParams):
    """Function that filters the input parameters. If the input parameter keys
    are no present on the "params" tuple, are not taken into account.
    """

    # active is a hook to disable the policy / action if needed
    params = ("element", "name", "vO", "elementType", "statusType", "status", "reason", "tokenOwner", "active")

    sanitizedParams = {}

    for key in params:
        if key in decisionParams:
            # We can get rid of this now
            # In CS names are with upper case, capitalize them here
            # sanitizedParams[ key[0].upper() + key[1:] ] = decisionParams[ key ]
            sanitizedParams[key] = decisionParams[key]

    return sanitizedParams


def _getComputingElementsByDomainName(targetDomain=None):
    """
    WARNING: TO ADD TO CSHelpers
    Gets all computing elements from /Resources/Sites/<>/<>/CE
    """

    _basePath = "Resources/Sites"
    ces = []

    domainNames = gConfig.getSections(_basePath)
    if not domainNames["OK"]:
        return S_ERROR("No domain names have been specified on the CS")
    domainNames = domainNames["Value"]

    unknownDomains = list(set(targetDomain) - set(domainNames))
    if unknownDomains:
        gLogger.warn("Domains %s belong to the policy parameters but not to the CS domains" % unknownDomains)

    knownDomains = list(set(domainNames) & set(targetDomain))
    if not knownDomains:
        gLogger.warn("Policy parameters domain names do not match with any CS domain names")
        return S_OK([])

    for domainName in knownDomains:
        gLogger.info("Fetching the list of Computing Elements belonging to domain %s" % domainName)
        domainSites = gConfig.getSections("%s/%s" % (_basePath, domainName))
        if not domainSites["OK"]:
            return domainSites
        domainSites = domainSites["Value"]

        for site in domainSites:
            siteCEs = gConfig.getSections("%s/%s/%s/CEs" % (_basePath, domainName, site))
            if not siteCEs["OK"]:
                # return siteCEs
                gLogger.error(siteCEs["Message"])
                continue
            siteCEs = siteCEs["Value"]
            ces.extend(siteCEs)

    # Remove duplicated ( just in case )
    ces = list(set(ces))
    gLogger.info("List of CEs: %s" % str(ces))

    return S_OK(ces)


def _filterPolicies(decisionParams, policyMatchParams):
    """
    Method that checks if the given policy doesn't meet certain conditions
    """
    elementType = decisionParams.get("elementType")
    name = decisionParams.get("name")

    # some policies may apply or not also depending on the VO's domain
    # 'CEAvailabilityPolicy' can be applied only if the CE is inside LCG
    if elementType and elementType.upper() == "CE" and "domain" in policyMatchParams:
        # WARNING: policyMatchParams['domain'] is a list of domains
        domains = policyMatchParams["domain"]
        result = _getComputingElementsByDomainName(targetDomain=domains)
        if result["OK"]:
            ces = result["Value"]
            # to verify that the given CE is in the list of the LCG CEs
            if name not in ces:
                gLogger.info("ComputingElement %s NOT found in domains %s" % (name, domains))
                return False
            else:
                gLogger.info("ComputingElement %s found in domains %s" % (name, domains))
        else:
            gLogger.warn("unable to verify if ComputingElement %s is in domains %s" % (name, domains))
            return False

    return True


def postProcessingPolicyList(policiesThatApply):
    """Put here any "hacky" post-processing"""

    # FIXME: the following 2 "if" are a "hack" for dealing with the following case:
    # an SE happens to be subject to, e.g., both the 'FreeDiskSpaceMB' and the 'FreeDiskSpaceGB' policies
    # (currently, there is no way to avoid that this happens, see e.g. LogSE)
    # When this is the case, supposing that an SE has 50 MB free, the policies evaluation will be the following:
    # - 'FreeDiskSpaceMB' will evaluate 'Active'
    # - 'FreeDiskSpaceGB' will evaluate 'Banned'
    # so the SE will end up being banned, but we want only the 'FreeDiskSpaceMB' to be considered.
    if ("FreeDiskSpaceMB", "FreeDiskSpaceMB", {}) in policiesThatApply:
        try:
            policiesThatApply.remove(("FreeDiskSpaceGB", "FreeDiskSpaceGB", {}))
        except ValueError:
            pass
        try:
            policiesThatApply.remove(("FreeDiskSpaceTB", "FreeDiskSpaceTB", {}))
        except ValueError:
            pass
    if ("FreeDiskSpaceGB", "FreeDiskSpaceGB", {}) in policiesThatApply:
        try:
            policiesThatApply.remove(("FreeDiskSpaceTB", "FreeDiskSpaceTB", {}))
        except ValueError:
            pass

    return policiesThatApply



""" Handler for logging in security.log.csv files
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import os

from DIRAC import gLogger, S_OK, S_ERROR, gConfig, rootPath
from DIRAC.Core.DISET.RequestHandler import RequestHandler
from DIRAC.Core.Utilities.File import mkDir
from DIRAC.FrameworkSystem.private.SecurityFileLog import SecurityFileLog
from DIRAC.FrameworkSystem.Client.SecurityLogClient import SecurityLogClient

__RCSID__ = "$Id$"

gSecurityFileLog = False


def initializeSecurityLoggingHandler(serviceInfo):
    global gSecurityFileLog

    serviceCS = serviceInfo["serviceSectionPath"]
    dataPath = gConfig.getValue("%s/DataLocation" % serviceCS, "data/securityLog")
    dataPath = dataPath.strip()
    if "/" != dataPath[0]:
        dataPath = os.path.realpath("%s/%s" % (gConfig.getValue("/LocalSite/InstancePath", rootPath), dataPath))
    gLogger.info("Data will be written into %s" % dataPath)
    mkDir(dataPath)

    try:
        testFile = "%s/seclog.jarl.test" % dataPath
        with open(testFile, "w"):
            pass
        os.unlink(testFile)
    except IOError:
        gLogger.fatal("Can't write to %s" % dataPath)
        return S_ERROR("Data location is not writable")
    # Define globals
    gSecurityFileLog = SecurityFileLog(dataPath)
    SecurityLogClient().setLogStore(gSecurityFileLog)
    return S_OK()


class SecurityLoggingHandler(RequestHandler):

    types_logAction = [(list, tuple)]

    def export_logAction(self, secMsg):
        """Log a single action"""
        result = gSecurityFileLog.logAction(secMsg)
        if not result["OK"]:
            return S_OK([(secMsg, result["Message"])])
        return S_OK()

    types_logActionBundle = [(list, tuple)]

    def export_logActionBundle(self, secMsgList):
        """Log a list of actions"""
        errorList = []
        for secMsg in secMsgList:
            result = gSecurityFileLog.logAction(secMsg)
            if not result["OK"]:
                errorList.append((secMsg, result["Message"]))
        if errorList:
            return S_OK(errorList)
        return S_OK()



""" This module hosts the logic for executing an RPC call.
"""
from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

__RCSID__ = "$Id$"

from DIRAC.Core.DISET.private.BaseClient import BaseClient
from DIRAC.Core.Utilities.ReturnValues import S_OK
from DIRAC.Core.Utilities.DErrno import cmpError, ENOAUTH


class InnerRPCClient(BaseClient):
    """This class instruments the BaseClient to perform RPC calls.
    At every RPC call, this class:

      * connects
      * proposes the action
      * sends the method parameters
      * retrieve the result
      * disconnect
    """

    # Number of times we retry the call.
    # The connection retry is handled by BaseClient
    __retry = 0

    def executeRPC(self, functionName, args):
        """Perform the RPC call, connect before and disconnect after.

        :param functionName: name of the function
        :param args: arguments to the function

        :return: in case of success, the return of the server call. In any case
                we add the connection stub to it.


        """
        retVal = self._connect()

        # Generate the stub which contains all the connection and call options
        # JSON: cast args to list for serialization purposes
        stub = [self._getBaseStub(), functionName, list(args)]
        if not retVal["OK"]:
            retVal["rpcStub"] = stub
            return retVal
        # Get the transport connection ID as well as the Transport object
        trid, transport = retVal["Value"]
        try:
            # Handshake to perform the RPC call for functionName
            retVal = self._proposeAction(transport, ("RPC", functionName))
            if not retVal["OK"]:
                if cmpError(retVal, ENOAUTH):  # This query is unauthorized
                    retVal["rpcStub"] = stub
                    return retVal
                else:  # we have network problem or the service is not responding
                    if self.__retry < 3:
                        self.__retry += 1
                        return self.executeRPC(functionName, args)
                    else:
                        retVal["rpcStub"] = stub
                        return retVal

            # Send the arguments to the function
            # Note: we need to convert the arguments to list
            # We do not need to deseralize it because variadic functions
            # can work with list too
            retVal = transport.sendData(S_OK(list(args)))
            if not retVal["OK"]:
                return retVal

            # Get the result of the call and append the stub to it
            # Note that the RPC timeout basically ticks here, since
            # the client waits for data for as long as the server side
            # processes the request.
            receivedData = transport.receiveData()
            if isinstance(receivedData, dict):
                receivedData["rpcStub"] = stub
            return receivedData
        finally:
            self._disconnect(trid)
